@@ -36,13 +36,11 @@ VALID_ACTIONS_PER_LIFECYCLE = {
 }
 
 
-BACKEND_URL = "https://back.sqreen.io"
-CHANGELOG_URL = "https://docs.sqreen.io/sqreen-for-python/python-changelog/"
-COMPATIBILITY_URL = (
-    "https://docs.sqreen.io/sqreen-for-python/python-agent-compatibility/"
-)
-INSTALLATION_URL = (
-    "https://docs.sqreen.io/sqreen-for-python/installing-the-python-agent/"
-)
-STATUS_URL = "http://status.sqreen.io/"
-TERMS_URL = "https://www.sqreen.io/terms.html"
+BACKEND_URL = "https://back.sqreen.com"
+CHANGELOG_URL = "https://docs.sqreen.com/python/release-notes/"
+COMPATIBILITY_URL = "https://docs.sqreen.com/python/compatibility/"
+LEGACY_BACKEND_URL = "https://back.sqreen.io"
+INGESTION_BACKEND_URL = "https://ingestion.sqreen.com/"
+INSTALLATION_URL = "https://docs.sqreen.com/python/installation/"
+STATUS_URL = "http://status.sqreen.com/"
+TERMS_URL = "https://www.sqreen.com/terms"
