@@ -6,6 +6,8 @@
 #
 
 from .client import ClientTracer
+from .consumer import ConsumerTracer
+from .producer import ProducerTracer
 from .server import ServerTracer
 
 
@@ -14,3 +16,5 @@ def init(interface_manager):
     """
     interface_manager.register(ClientTracer())
     interface_manager.register(ServerTracer())
+    interface_manager.register(ProducerTracer())
+    interface_manager.register(ConsumerTracer())
