@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -7,17 +7,50 @@
 """ Instrumentation helper responsible for adding dynamic callback
 """
 import logging
-import sys
 
 from ..remote_exception import RemoteException
 from ..utils import HAS_ASYNCIO
 from .import_hook import ModuleFinder
-from .strategies import (DBApi2Strategy, DjangoStrategy, FlaskStrategy,
-                         ImportHookStrategy, Psycopg2Strategy, PyramidStrategy)
+from .strategies import (
+    DBApi2Strategy,
+    DjangoStrategy,
+    FlaskStrategy,
+    ImportHookStrategy,
+    LXMLResolverStrategy,
+    Psycopg2Strategy,
+    PyramidStrategy,
+    WSGIFactoryStrategy,
+    WSGIReceiverStrategy,
+    WSGIStrategy,
+)
+
+STRATEGIES = {
+    "import_hook": ImportHookStrategy,
+    "DBApi2": DBApi2Strategy,
+    "django": DjangoStrategy,
+    "psycopg2": Psycopg2Strategy,
+    "flask": FlaskStrategy,
+    "pyramid": PyramidStrategy,
+    "wsgi": WSGIStrategy,
+    "wsgi_factory": WSGIFactoryStrategy,
+    "wsgi_receiver": WSGIReceiverStrategy,
+    "lxml_resolver": LXMLResolverStrategy,
+}
 
 if HAS_ASYNCIO:
-    from .strategies import (AioHTTPHookStrategy, AioHTTPInstallStrategy,
-                             AsyncEventLoopStrategy, AsyncImportHookStrategy)
+    from .strategies import (
+        AioHTTPHookStrategy,
+        AioHTTPInstallStrategy,
+        AsyncEventLoopStrategy,
+        AsyncImportHookStrategy,
+    )
+
+    STRATEGIES.update({
+        "async_event_loop": AsyncEventLoopStrategy,
+        "async_import_hook": AsyncImportHookStrategy,
+        "aiohttp_install": AioHTTPInstallStrategy,
+        "aiohttp_hook": AioHTTPHookStrategy,
+    })
 
 LOGGER = logging.getLogger(__name__)
 
@@ -49,6 +82,7 @@ class Instrumentation(object):
         self.strategies = {}
         self.before_hook_point = before_hook_point
         self.import_hook = ModuleFinder()
+        self.enabled = False
 
     def add_callback(self, callback):
         """ Add a callback. The callback defines itself where it should
@@ -63,24 +97,26 @@ class Instrumentation(object):
             # Check if we already have a strategy
             if strategy_id in self.strategies:
                 strategy_instance = self.strategies[strategy_id]
-                LOGGER.debug("Reusing strategy %s for id %s",
-                             strategy_instance, strategy_id)
             else:
-                strategy_instance = strategy_class(strategy_id, self.observation_queue, self.queue,
-                                                   self.import_hook, self.before_hook_point)
-                LOGGER.debug("Instantiate strategy %s for id %s -> %s",
-                             strategy_class, strategy_id, strategy_instance)
+                strategy_instance = strategy_class(
+                    strategy_id,
+                    self.observation_queue,
+                    self.queue,
+                    self.import_hook,
+                    self.before_hook_point,
+                )
                 self.strategies[strategy_id] = strategy_instance
-
             strategy_instance.add_callback(callback)
         except Exception:
             # If the strategy fails to hook either at instantiation
             # or hook method, catch the exception and log it
-            LOGGER.exception("Callback %r fails to hook", callback, exc_info=True)
+            LOGGER.debug(
+                "Callback %r fails to hook", callback, exc_info=True
+            )
 
             # And send the exception back to the backend
-            infos = {'callback': callback.__dict__}
-            remote_exception = RemoteException(sys.exc_info(), infos)
+            infos = {"callback": callback.__dict__}
+            remote_exception = RemoteException.from_exc_info(callback_payload=infos)
             self.queue.put(remote_exception)
 
     def deinstrument(self, callback):
@@ -99,6 +135,7 @@ class Instrumentation(object):
         """
         for strategy in self.strategies.values():
             strategy.deinstrument_all()
+        self.enabled = False
 
     def hook_all(self):
         """ First, inject the import hook in sys.meta if not present.
@@ -112,6 +149,7 @@ class Instrumentation(object):
         for strategy in self.strategies.values():
             strategy.hook()
         self.import_hook.apply_patchers()
+        self.enabled = True
 
     @staticmethod
     def _get_strategy_class(strategy):
@@ -119,26 +157,8 @@ class Instrumentation(object):
         in parameter.
         Raise a NotImplementedError if the strategy is unknown.
         """
-        if strategy == 'import_hook':
-            return ImportHookStrategy
-        elif strategy == 'DBApi2':
-            return DBApi2Strategy
-        elif strategy == 'django':
-            return DjangoStrategy
-        elif strategy == 'psycopg2':
-            return Psycopg2Strategy
-        elif strategy == 'flask':
-            return FlaskStrategy
-        elif strategy == 'pyramid':
-            return PyramidStrategy
-        elif HAS_ASYNCIO and strategy == 'async_event_loop':
-            return AsyncEventLoopStrategy
-        elif HAS_ASYNCIO and strategy == 'async_import_hook':
-            return AsyncImportHookStrategy
-        elif HAS_ASYNCIO and strategy == 'aiohttp_install':
-            return AioHTTPInstallStrategy
-        elif HAS_ASYNCIO and strategy == 'aiohttp_hook':
-            return AioHTTPHookStrategy
-        else:
-            err_msg = "Unknown hooking_strategy {}"
-            raise NotImplementedError(err_msg.format(strategy))
+        strategy_cls = STRATEGIES.get(strategy)
+        if strategy_cls is None:
+            raise NotImplementedError(
+                "Unknown hooking strategy {!r}".format(strategy))
+        return strategy_cls
