@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -7,6 +7,7 @@
 """ Base strategy
 """
 import logging
+from bisect import bisect
 from collections import defaultdict
 
 from ..hook_point import LIFECYCLE_METHODS
@@ -22,7 +23,9 @@ class BaseStrategy(object):
     directly.
     """
 
-    def __init__(self, observation_queue, queue, import_hook, before_hook_point=None):
+    def __init__(
+        self, observation_queue, queue, import_hook, before_hook_point=None
+    ):
         self.observation_queue = observation_queue
         self.queue = queue
         self.unique_callbacks = set()
@@ -40,9 +43,20 @@ class BaseStrategy(object):
         """
         key = (callback.hook_module, callback.hook_name)
 
-        for method in LIFECYCLE_METHODS.values():
+        # Normal order insertion
+        if hasattr(callback, LIFECYCLE_METHODS["PRE"]):
+            callbacks = self.callbacks[key][LIFECYCLE_METHODS["PRE"]]
+            idx = bisect([c.priority for c in callbacks], callback.priority)
+            callbacks.insert(idx, callback)
+
+        # Reverse order insertion
+        for method in (LIFECYCLE_METHODS["POST"], LIFECYCLE_METHODS["FAILING"]):
             if hasattr(callback, method):
-                self.callbacks[key][method].append(callback)
+                rev_callbacks = list(reversed(self.callbacks[key][method]))
+                idx = bisect([c.priority for c in rev_callbacks], callback.priority)
+                rev_callbacks.insert(idx, callback)
+                rev_callbacks.reverse()
+                self.callbacks[key][method] = rev_callbacks
 
         self.unique_callbacks.add(callback)
 
@@ -53,19 +67,19 @@ class BaseStrategy(object):
         """ Returns callbacks with a pre lifecycle method for the key
         (hook_module, hook_name)
         """
-        return self.callbacks[key][LIFECYCLE_METHODS['PRE']]
+        return self.callbacks[key][LIFECYCLE_METHODS["PRE"]]
 
     def get_post_callbacks(self, key):
         """ Returns callbacks with a post lifecycle method for the key
         (hook_module, hook_name)
         """
-        return self.callbacks[key][LIFECYCLE_METHODS['POST']]
+        return self.callbacks[key][LIFECYCLE_METHODS["POST"]]
 
     def get_failing_callbacks(self, key):
         """ Returns callbacks with a failing lifecycle method for the key
         (hook_module, hook_name)
         """
-        return self.callbacks[key][LIFECYCLE_METHODS['FAILING']]
+        return self.callbacks[key][LIFECYCLE_METHODS["FAILING"]]
 
     def _total_callbacks(self):
         """ Count the total number of callbacks
@@ -97,10 +111,14 @@ class BaseStrategy(object):
         except KeyError:
             pass
 
-        LOGGER.debug("Number of remaining callback for %s: %s", self, self._total_callbacks())
+        LOGGER.debug(
+            "Number of remaining callback for %s: %s",
+            self,
+            self._total_callbacks(),
+        )
 
         # If no more callbacks are set deinstrument
-        if self._total_callbacks() is 0:
+        if self._total_callbacks() == 0:
             self._restore()
 
     def deinstrument_all(self):
