@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -9,7 +9,7 @@
 
 from logging import getLogger
 
-from ..middlewares import FlaskMiddleware
+from ..middlewares.flask_middleware import FlaskMiddleware
 from .framework import FrameworkStrategy
 
 LOGGER = getLogger(__name__)
@@ -30,9 +30,21 @@ class FlaskStrategy(FrameworkStrategy):
     HOOK_CLASS = "Flask"
     HOOK_METHOD = "full_dispatch_request"
 
-    def __init__(self, strategy_key, observation_queue, queue, import_hook, before_hook_point=None):
-        super(FlaskStrategy, self).__init__(strategy_key, observation_queue, queue,
-                                            import_hook, before_hook_point)
+    def __init__(
+        self,
+        strategy_key,
+        observation_queue,
+        queue,
+        import_hook,
+        before_hook_point=None,
+    ):
+        super(FlaskStrategy, self).__init__(
+            strategy_key,
+            observation_queue,
+            queue,
+            import_hook,
+            before_hook_point,
+        )
 
         self.middleware = FlaskMiddleware(self, observation_queue, queue)
         self.wrapper = replace_wsgi_app
