@@ -6,10 +6,14 @@
 #
 import logging
 from collections import defaultdict, deque
+from datetime import timedelta
+from itertools import groupby
+from operator import itemgetter
 
 from .events import RequestRecord
 from .performance_metrics import RequestTrace, Trace
 from .runner import MetricsEvent
+from .utils import now
 
 LOGGER = logging.getLogger(__name__)
 
@@ -35,6 +39,7 @@ class RequestRecorder:
         self.traces.clear()
         self.identity = None
         self.request_trace = None
+        self.request_overtime = False
 
     def trace(self, name=None):
         """Start a new trace from a request trace."""
@@ -64,6 +69,17 @@ class RequestRecorder:
         if self.request_trace is not None:
             return self.request_trace.aggregated_duration_ms
 
+    def record_overtime(self, name, at):
+        """Record which callback triggered the overtime."""
+        if not self.request_overtime and self.request_trace is not None:
+            at = now() if at is None else at
+            self.observe(
+                "observations",
+                ("request_overbudget_cb", at, name, 1),
+                report=False
+            )
+            self.request_overtime = True
+
     def observe(self, what, observation, payload_sections=None, report=True):
         """Record an observation."""
         if what == "sdk":
@@ -106,23 +122,31 @@ class RequestRecorder:
 
     def _put_metrics(self, queue, observation_queue):
         if LOGGER.isEnabledFor(logging.DEBUG):
-            metrics = ["\t{0}: {3:.02f}".format(*i) for i in self.traces]
-            LOGGER.debug("Performance metrics:\n%s", "\n".join(metrics))
+            it = groupby(self.traces, key=itemgetter(0))
+            metrics = []
+            for key, group in it:
+                group = list(group)
+                line = ""
+                if key.startswith("sq."):
+                    approx_date = \
+                        group[0][1] - timedelta(seconds=group[0][3] / 1000.0)
+                    line += str(approx_date)
+                line += "\t{0}: ".format(key)
+                line += " ".join("{0:.02f}".format(i[3]) for i in group)
+                if len(group) > 1:
+                    line += " ({:.02f} total)".format(sum(i[3] for i in group))
+                metrics.append(line)
+            if self.request_overtime:
+                metrics.append("\trequest was overbudget")
+            if metrics:
+                LOGGER.debug("Performance metrics:\n%s", "\n".join(metrics))
 
         # Send the performance metrics to observation queue for aggregation
         while self.traces:
             observation_queue.put(self.traces.popleft())
 
-        # Pop the overtime callback observations so that they don't appear
-        # in the request record when it is reported and only send the
-        # first one for aggregation.
-        overtime_callbacks = self.observations.pop("overtime_callbacks", [])
-        if overtime_callbacks:
-            observation_queue.put(overtime_callbacks[0])
-
-        # Aggregate observations if the request is not reported
-        if not self.report:
-            for observation in self.observations.get("observations", []):
-                observation_queue.put(observation)
+        # Aggregate metric observations
+        for observation in self.observations.pop("observations", []):
+            observation_queue.put(observation)
         if observation_queue.half_full():
             queue.put(MetricsEvent)
