@@ -59,9 +59,15 @@ class SDKAuthMetrics(AuthMetricsMixin, RuleCallback):
 
     def pre(self, instance, args, kwargs, **options):
         """Report a metric for SDK authentication tracking."""
-        if args and kwargs:
-            auth_status = "sdk-login-success" if args[0] else "sdk-login-fail"
-            self.record_auth_metrics(auth_status, kwargs)
+
+        def _extract(success, **user_identifiers):
+            return success, user_identifiers
+
+        success, identifiers = _extract(*args, **kwargs)
+
+        if identifiers:
+            auth_status = "sdk-login-success" if success else "sdk-login-fail"
+            self.record_auth_metrics(auth_status, identifiers)
 
 
 class SDKSignupMetrics(AuthMetricsMixin, RuleCallback):
