@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -15,10 +15,13 @@ LOGGER = logging.getLogger(__name__)
 
 class CountHTTPCodesCBDjango(RuleCallback):
 
-    def post(self, original, response, *args, **kwargs):
+    INTERRUPTIBLE = False
+
+    def post(self, instance, args, kwargs, result=None, **options):
         """ Recover the status code and update the http_code metric
         """
+        response = result
         status_code = response.status_code
-        self.record_observation('http_code', str(status_code), 1)
+        self.record_observation("http_code", str(status_code), 1)
 
         return {}
