@@ -4,7 +4,8 @@
 #
 #     https://www.sqreen.io/terms.html
 #
-""" Look for known crawlers user-agents
+""" Store a Django request and response in the storage and clear the request
+context.
 """
 from logging import getLogger
 
