@@ -16,9 +16,11 @@ LOGGER = getLogger(__name__)
 
 class RecordRequestContextFlask(RecordRequestContext):
     def pre(self, instance, args, kwargs, **options):
-        from flask import request
+        from flask import _request_ctx_stack
 
-        self._store_request(FlaskRequest(request))
+        reqctx = _request_ctx_stack.top
+        if reqctx is not None:
+            self._store_request(FlaskRequest(reqctx.request))
 
     def post(self, instance, args, kwargs, result=None, **options):
         self.storage.store_response(FlaskResponse(result))
