@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -7,18 +7,16 @@
 """ Look for injected shell vars
 """
 import os
-from collections import Mapping
 from logging import getLogger
 
-from ..utils import is_string
+from ..utils import Mapping, is_string
 from .regexp_rule import RegexpRule
 
 LOGGER = getLogger(__name__)
 
 
 class ShellEnvCB(RegexpRule):
-
-    def pre(self, original, *args, **kwargs):
+    def pre(self, instance, args, kwargs, **options):
         """
         Examine environment variables to find shellshock injections.
         Works if env is passed in positional or keyword argument.
@@ -47,7 +45,7 @@ class ShellEnvCB(RegexpRule):
         if not found:
             return
 
-        infos = {'variable_name': var, 'variable_value': val, 'found': found}
+        infos = {"variable_name": var, "variable_value": val, "found": found}
         self.record_attack(infos)
 
-        return {'status': 'raise', 'data': found, 'rule_name': self.rule_name}
+        return {"status": "raise", "data": found, "rule_name": self.rule_name}
