@@ -184,6 +184,16 @@ def get_session(url, token, app_name=None, proxy_url=None, use_legacy_url=False)
     return session
 
 
+def get_proxy_url():
+    """ Returns the HTTP proxy url or None if not set.
+    """
+    legacy_name = CONFIG["PROXY_URL"]
+    new_name = CONFIG["HTTP_PROXY"]
+    if legacy_name and new_name:
+        LOGGER.warning("Both proxy_url and http_proxy are set, using http_proxy.")
+    return new_name or legacy_name
+
+
 def post_app_sqreen_exception(data):
     """Post a Sqreen exception happening at application level.
     This function does not need a session. It always uses the legacy backend URL
@@ -199,7 +209,7 @@ def post_app_sqreen_exception(data):
     try:
         app_name = get_app_name()
         session = get_session(
-            CONFIG["LEGACY_URL"], token, app_name, proxy_url=CONFIG["PROXY_URL"],
+            CONFIG["LEGACY_URL"], token, app_name, proxy_url=get_proxy_url(),
             use_legacy_url=True
         )
         session.post_app_sqreen_exception(data)
@@ -492,7 +502,7 @@ class RunnerThread(Thread):
                 if not CONFIG["NO_SNIFF_DOMAINS"]:
                     # Detect if the configured URLs are reachable.
                     use_legacy_url = not sqreen_compatible_backends(
-                        [CONFIG["URL"], CONFIG["INGESTION_URL"]], proxy_url=CONFIG["PROXY_URL"])
+                        [CONFIG["URL"], CONFIG["INGESTION_URL"]], proxy_url=get_proxy_url())
                 else:
                     # Force use of the configured URLs.
                     use_legacy_url = False
@@ -501,7 +511,7 @@ class RunnerThread(Thread):
                 # if the new URLs are not available.
                 session_url = CONFIG["LEGACY_URL"] if use_legacy_url else CONFIG["URL"]
                 session = get_session(
-                    session_url, token, app_name, proxy_url=CONFIG["PROXY_URL"],
+                    session_url, token, app_name, proxy_url=get_proxy_url(),
                     use_legacy_url=use_legacy_url
                 )
 
