@@ -6,11 +6,11 @@
 #
 """ Various utils
 """
+import datetime
 import json
 import sys
 import types
 from collections import deque
-from datetime import datetime, timedelta
 from inspect import isclass
 from itertools import islice
 from logging import getLogger
@@ -49,6 +49,8 @@ HAS_ASYNCIO = sys.version_info >= (3, 4)
 
 NONE_TYPE = type(None)
 
+ZERO_TD = datetime.timedelta(0)
+
 if sys.version_info[0] < 3:
     ALL_STRING_CLASS = basestring  # noqa
     STRING_CLASS = str
@@ -63,6 +65,24 @@ if sys.version_info[0] < 3:
     def create_bound_method(func, obj):
         return types.MethodType(func, obj, obj.__class__)
 
+    class FixedOffsetTZ(datetime.tzinfo):
+        """Fixed offset in minutes east from UTC."""
+
+        def __init__(self, offset, name):
+            self.__offset = datetime.timedelta(minutes=offset)
+            self.__name = name
+
+        def utcoffset(self, dt):
+            return self.__offset
+
+        def tzname(self, dt):
+            return self.__name
+
+        def dst(self, dt):
+            return ZERO_TD
+
+    UTC = FixedOffsetTZ(0, "UTC")
+
 else:
     ALL_STRING_CLASS = str
     STRING_CLASS = str
@@ -76,6 +96,10 @@ else:
 
     create_bound_method = types.MethodType
 
+    FixedOffsetTZ = datetime.timezone
+
+    UTC = datetime.timezone.utc
+
 
 def is_string(value):  # type: (Any) -> bool
     """ Check if a value is a valid string, compatible with python 2 and python 3
@@ -133,11 +157,14 @@ def to_unicode_safe(value):  # type: (Any) -> Optional[str]
     if isinstance(value, bytes):
         return value.decode("utf-8", errors="replace")
     # If the value is not already a unicode string
-    # Try to convert it to a unicode string
+    # Try to convert it to a string
     # by calling the standard Python __str__ method on
     # the value.
     elif not is_unicode(value):
-        return UNICODE_CLASS(value)
+        value = STRING_CLASS(value)
+        # In Python 2.7, the returned value has no encoding
+        if isinstance(value, bytes):
+            return value.decode("utf-8", errors="replace")
     # Value is already a unicode string
     return value
 
@@ -296,13 +323,20 @@ def ip_network(address, strict=True):
     return _ip_network(UNICODE_CLASS(address), strict=strict)
 
 
-def truncate_time(dt=None, round_to=60):  # type: (datetime, int) -> datetime
+def truncate_time(dt=None, round_to=60):  # type: (Optional[datetime.datetime], int) -> datetime.datetime
     """Return a datetime rounded to the previous given second interval."""
     if dt is None:
-        dt = datetime.utcnow()
+        dt = now()
     dt = dt.replace(microsecond=0)
-    seconds = (dt - dt.min).total_seconds()
-    return dt - timedelta(seconds=seconds % round_to)
+    if dt.tzinfo is None:
+        naive_dt = dt
+    else:
+        offset = dt.utcoffset()
+        if offset is None:
+            offset = ZERO_TD
+        naive_dt = dt.replace(tzinfo=None) - offset
+    seconds = (naive_dt - naive_dt.min).total_seconds()
+    return dt - datetime.timedelta(seconds=seconds % round_to)
 
 
 def flatten(iterable, max_iterations=1000):
@@ -338,3 +372,141 @@ def flatten(iterable, max_iterations=1000):
             values.append(iterable)
 
     return keys, values
+
+
+def naive_dt_to_utc(dt):  # type: (datetime.datetime) -> datetime.datetime
+    """Convert naive datetime to timezone aware datetime.
+    By default, we consider all naive datetime to be in UTC.
+    """
+    if dt.tzinfo is None:
+        return dt.replace(tzinfo=UTC)
+    return dt
+
+
+def now():  # type: () -> datetime.datetime
+    """ Return the current UTC time.
+    """
+    return datetime.datetime.now(UTC)
+
+
+# Helpers for parsing the result of isoformat() from
+# https://github.com/python/cpython/blob/3.8/Lib/datetime.py
+def _parse_isoformat_date(dtstr):
+    # It is assumed that this function will only be called with a
+    # string of length exactly 10, and (though this is not used) ASCII-only
+    year = int(dtstr[0:4])
+    if dtstr[4] != '-':
+        raise ValueError('Invalid date separator: %s' % dtstr[4])
+
+    month = int(dtstr[5:7])
+
+    if dtstr[7] != '-':
+        raise ValueError('Invalid date separator')
+
+    day = int(dtstr[8:10])
+
+    return [year, month, day]
+
+
+def _parse_hh_mm_ss_ff(tstr):
+    # Parses things of the form HH[:MM[:SS[.fff[fff]]]]
+    len_str = len(tstr)
+
+    time_comps = [0, 0, 0, 0]
+    pos = 0
+    for comp in range(0, 3):
+        if (len_str - pos) < 2:
+            raise ValueError('Incomplete time component')
+
+        time_comps[comp] = int(tstr[pos:pos + 2])
+
+        pos += 2
+        next_char = tstr[pos:pos + 1]
+
+        if not next_char or comp >= 2:
+            break
+
+        if next_char != ':':
+            raise ValueError('Invalid time separator: %c' % next_char)
+
+        pos += 1
+
+    if pos < len_str:
+        if tstr[pos] != '.':
+            raise ValueError('Invalid microsecond component')
+        else:
+            pos += 1
+
+            len_remainder = len_str - pos
+            if len_remainder not in (3, 6):
+                raise ValueError('Invalid microsecond component')
+
+            time_comps[3] = int(tstr[pos:])
+            if len_remainder == 3:
+                time_comps[3] *= 1000
+
+    return time_comps
+
+
+def _parse_isoformat_time(tstr):
+    # Format supported is HH[:MM[:SS[.fff[fff]]]][+HH:MM[:SS[.ffffff]]]
+    len_str = len(tstr)
+    if len_str < 2:
+        raise ValueError('Isoformat time too short')
+
+    # This is equivalent to re.search('[+-]', tstr), but faster
+    tz_pos = (tstr.find('-') + 1 or tstr.find('+') + 1 or tstr.find('Z') + 1)
+    timestr = tstr[:tz_pos - 1] if tz_pos > 0 else tstr
+
+    time_comps = _parse_hh_mm_ss_ff(timestr)
+
+    tzi = None
+    if tstr[tz_pos - 1] == 'Z':
+        tzi = UTC
+    elif tz_pos > 0:
+        tzstr = tstr[tz_pos:]
+
+        # Valid time zone strings are:
+        # HH:MM               len: 5
+        # HH:MM:SS            len: 8
+        # HH:MM:SS.ffffff     len: 15
+
+        if len(tzstr) not in (5, 8, 15):
+            raise ValueError('Malformed time zone string')
+
+        tz_comps = _parse_hh_mm_ss_ff(tzstr)
+        if all(x == 0 for x in tz_comps):
+            tzi = UTC
+        else:
+            tzsign = -1 if tstr[tz_pos - 1] == '-' else 1
+
+            td = datetime.timedelta(
+                hours=tz_comps[0], minutes=tz_comps[1],
+                seconds=tz_comps[2], microseconds=tz_comps[3])
+
+            tzi = FixedOffsetTZ(tzsign * td, tzstr)
+
+    time_comps.append(tzi)
+
+    return time_comps
+
+
+def datetime_from_isoformat(date_string):
+    # Split this at the separator
+    dstr = date_string[0:10]
+    tstr = date_string[11:]
+
+    try:
+        date_components = _parse_isoformat_date(dstr)
+    except ValueError:
+        raise ValueError('Invalid isoformat string: {!r}'.format(date_string))
+
+    if tstr:
+        try:
+            time_components = _parse_isoformat_time(tstr)
+        except ValueError:
+            raise ValueError('Invalid isoformat string: {!r}'.format(date_string))
+    else:
+        time_components = [0, 0, 0, 0, None]
+
+    return datetime.datetime(*(date_components + time_components))
